@@ -1,5 +1,4 @@
-#ifndef STAT_H
-#define STAT_H
+#pragma once
 
 #include <QMap>
 #include <QVector>
@@ -45,6 +44,8 @@ class Stat {
         }
     }
 
+    // TODO make this an enum class for improved typesafety and issues with
+    // ambiguous overloads.
     enum ComputeTypes {
         NONE            = 0x0000,
         // O(1) in time and space.
@@ -75,7 +76,7 @@ class Stat {
         // Used for marking stats recorded in BASE mode.
         STATS_BASE        = 0x1000,
     };
-    typedef int ComputeFlags;
+    Q_DECLARE_FLAGS(ComputeFlags, ComputeTypes);
 
     static Experiment::Mode modeFromFlags(ComputeFlags flags) {
         if (flags & Stat::STATS_EXPERIMENT) {
@@ -118,20 +119,29 @@ class Stat {
     double m_variance_sk;
     QMap<double, double> m_histogram;
 
-    static bool track(const QString& tag,
+    static bool track(QString tag,
                       Stat::StatType type,
                       Stat::ComputeFlags compute,
                       double value);
+
+    // Disallow to use this class with implicit converted char strings.
+    // This should not be uses to avoid unicode encoding and memory
+    // allocation at every call. Use a static tag like this:
+    // static const QString tag("TAG TEXT");
+    static bool track(const char *,
+                      Stat::StatType,
+                      Stat::ComputeFlags,
+                      double) = delete;
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(Stat::ComputeFlags);
+
 QDebug operator<<(QDebug dbg, const Stat &stat);
 
 struct StatReport {
-    char* tag;
+    QString tag;
     qint64 time;
     Stat::StatType type;
     Stat::ComputeFlags compute;
     double value;
 };
-
-#endif /* STAT_H */
