@@ -1,36 +1,39 @@
-/**
- * @file vinylcontrolmanager.cpp
- * @author Bill Good <bkgood@gmail.com>
- * @date April 15, 2011
- */
-
-#include "controlobject.h"
-#include "controlobjectslave.h"
-#include "controlobjectthread.h"
-#include "playermanager.h"
-#include "soundmanager.h"
-#include "util/timer.h"
-#include "vinylcontrol/defs_vinylcontrol.h"
-#include "vinylcontrol/vinylcontrol.h"
-#include "vinylcontrol/vinylcontrolprocessor.h"
-#include "vinylcontrol/vinylcontrolxwax.h"
-
 #include "vinylcontrol/vinylcontrolmanager.h"
 
+#include <QRegularExpression>
+
+#include "audio/types.h"
+#include "control/controlobject.h"
+#include "control/controlproxy.h"
+#include "mixer/playermanager.h"
+#include "moc_vinylcontrolmanager.cpp"
+#include "soundio/soundmanager.h"
+#include "util/defs.h"
+#include "vinylcontrol/defs_vinylcontrol.h"
+#include "vinylcontrol/vinylcontrolprocessor.h"
+
+namespace {
+const QRegularExpression kChannelRegex(QStringLiteral("\\[Channel([1-9]\\d*)\\]"));
+} // namespace
+
 VinylControlManager::VinylControlManager(QObject* pParent,
-                                         ConfigObject<ConfigValue>* pConfig,
+                                         UserSettingsPointer pConfig,
                                          SoundManager* pSoundManager)
         : QObject(pParent),
           m_pConfig(pConfig),
           m_pProcessor(new VinylControlProcessor(this, pConfig)),
           m_iTimerId(-1),
-          m_pNumDecks(NULL),
+          m_pNumDecks(nullptr),
           m_iNumConfiguredDecks(0) {
     // Register every possible VC input with SoundManager to route to the
     // VinylControlProcessor.
     for (int i = 0; i < kMaximumVinylControlInputs; ++i) {
         pSoundManager->registerInput(
-            AudioInput(AudioInput::VINYLCONTROL, 0, 0, i), m_pProcessor);
+                AudioInput(AudioPathType::VinylControl,
+                        0,
+                        mixxx::audio::ChannelCount::stereo(),
+                        i),
+                m_pProcessor);
     }
 }
 
@@ -41,22 +44,31 @@ VinylControlManager::~VinylControlManager() {
     // turn off vinyl control so it won't be enabled on load (this is redundant to mixxx.cpp)
     for (int i = 0; i < m_iNumConfiguredDecks; ++i) {
         QString group = PlayerManager::groupForDeck(i);
-        m_pConfig->set(ConfigKey(group, "vinylcontrol_enabled"), false);
-        m_pConfig->set(ConfigKey(VINYL_PREF_KEY, QString("cueing_ch%1").arg(i + 1)),
-            ConfigValue(static_cast<int>(ControlObject::get(
-                ConfigKey(group, "vinylcontrol_cueing")))));
-        m_pConfig->set(ConfigKey(VINYL_PREF_KEY, QString("mode_ch%1").arg(i + 1)),
-            ConfigValue(static_cast<int>(ControlObject::get(
-                ConfigKey(group, "vinylcontrol_mode")))));
+        m_pConfig->setValue(ConfigKey(group, "vinylcontrol_enabled"), false);
+        m_pConfig->set(ConfigKey(VINYL_PREF_KEY, QStringLiteral("cueing_ch%1").arg(i + 1)),
+                ConfigValue(static_cast<int>(ControlObject::get(
+                        ConfigKey(group, "vinylcontrol_cueing")))));
+        m_pConfig->set(ConfigKey(VINYL_PREF_KEY, QStringLiteral("mode_ch%1").arg(i + 1)),
+                ConfigValue(static_cast<int>(ControlObject::get(
+                        ConfigKey(group, "vinylcontrol_mode")))));
     }
 }
 
 void VinylControlManager::init() {
-    m_pNumDecks = new ControlObjectSlave("[Master]", "num_decks", this);
-    m_pNumDecks->connectValueChanged(SLOT(slotNumDecksChanged(double)));
+    m_pNumDecks = new ControlProxy(QStringLiteral("[App]"), QStringLiteral("num_decks"), this);
+    m_pNumDecks->connectValueChanged(this, &VinylControlManager::slotNumDecksChanged);
     slotNumDecksChanged(m_pNumDecks->get());
 }
 
+void VinylControlManager::toggleVinylControl(int deck) {
+    if (deck < 0 || deck >= m_pVcEnabled.size()) {
+        return;
+    }
+
+    ControlProxy* pEnabled = m_pVcEnabled[deck];
+    pEnabled->set(!pEnabled->toBool());
+}
+
 void VinylControlManager::slotNumDecksChanged(double dNumDecks) {
     int num_decks = static_cast<int>(dNumDecks);
 
@@ -75,24 +87,33 @@ void VinylControlManager::slotNumDecksChanged(double dNumDecks) {
 
     for (int i = m_iNumConfiguredDecks; i < num_decks; ++i) {
         QString group = PlayerManager::groupForDeck(i);
-        m_pVcEnabled.push_back(new ControlObjectThread(group, "vinylcontrol_enabled", this));
-        m_pVcEnabled.back()->set(0);
+        ControlProxy* pEnabled = new ControlProxy(group, "vinylcontrol_enabled", this);
+        m_pVcEnabled.push_back(pEnabled);
+        pEnabled->connectValueChanged(this, [this, i] { slotVinylControlEnabledChanged(i); });
 
         // Default cueing should be off.
         ControlObject::set(ConfigKey(group, "vinylcontrol_cueing"),
-                           m_pConfig->getValueString(ConfigKey(
-                                   VINYL_PREF_KEY,
-                                   QString("cueing_ch%1").arg(i + 1)), "0").toDouble());
+                m_pConfig->getValue(
+                        ConfigKey(VINYL_PREF_KEY, QString("cueing_ch%1").arg(i + 1)),
+                        0.0));
         // Default mode should be relative.
-        const QString kDefaultMode = QString::number(MIXXX_VCMODE_RELATIVE);
         ControlObject::set(ConfigKey(group, "vinylcontrol_mode"),
-                           m_pConfig->getValueString(ConfigKey(
-                                   VINYL_PREF_KEY,
-                                   QString("mode_ch%1").arg(i + 1)), kDefaultMode).toDouble());
+                m_pConfig->getValue(
+                        ConfigKey(VINYL_PREF_KEY, QString("mode_ch%1").arg(i + 1)),
+                        MIXXX_VCMODE_RELATIVE));
     }
     m_iNumConfiguredDecks = num_decks;
 }
 
+void VinylControlManager::slotVinylControlEnabledChanged(int deck) {
+    VERIFY_OR_DEBUG_ASSERT(deck >= 0 && deck < m_pVcEnabled.size()) {
+        return;
+    }
+
+    ControlProxy* pEnabled = m_pVcEnabled.at(deck);
+    emit vinylControlDeckEnabled(deck, pEnabled->toBool());
+}
+
 void VinylControlManager::requestReloadConfig() {
     m_pProcessor->requestReloadConfig();
 }
@@ -102,17 +123,17 @@ bool VinylControlManager::vinylInputConnected(int deck) {
         return false;
     }
     if (deck < 0 || deck >= m_pVcEnabled.length()) {
-        qDebug() << "WARNING, tried to get vinyl enabled status for non-existant deck " << deck;
+        qDebug() << "WARNING, tried to get vinyl enabled status for non-existent deck " << deck;
         return false;
     }
     return m_pProcessor->deckConfigured(deck);
 }
 
 int VinylControlManager::vinylInputFromGroup(const QString& group) {
-    QRegExp channelMatcher("\\[Channel([1-9]\\d*)\\]");
-    if (channelMatcher.exactMatch(group)) {
+    QRegularExpressionMatch channelMatch = kChannelRegex.match(group);
+    if (channelMatch.hasMatch()) {
         bool ok = false;
-        int input = channelMatcher.cap(1).toInt(&ok);
+        int input = channelMatch.captured(1).toInt(&ok);
         return ok ? input - 1 : -1;
     }
     return -1;
@@ -140,7 +161,7 @@ void VinylControlManager::removeSignalQualityListener(VinylSignalQualityListener
 
 void VinylControlManager::updateSignalQualityListeners() {
     FIFO<VinylSignalQualityReport>* signalQualityFifo = m_pProcessor->getSignalQualityFifo();
-    if (signalQualityFifo == NULL) {
+    if (signalQualityFifo == nullptr) {
         return;
     }
 
