@@ -1,11 +1,10 @@
-#ifndef WAVEFORMRENDERPREROLL_H
-#define WAVEFORMRENDERPREROLL_H
+#pragma once
 
 #include <QColor>
 
-#include "util.h"
+#include "skin/legacy/skincontext.h"
+#include "util/class.h"
 #include "waveform/renderers/waveformrendererabstract.h"
-#include "skin/skincontext.h"
 
 class WaveformRendererPreroll : public WaveformRendererAbstract {
   public:
@@ -17,8 +16,6 @@ class WaveformRendererPreroll : public WaveformRendererAbstract {
 
   private:
     QColor m_color;
-
+    
     DISALLOW_COPY_AND_ASSIGN(WaveformRendererPreroll);
 };
-
-#endif /* WAVEFORMRENDERPREROLL_H */
