@@ -1,12 +1,13 @@
+#include "waveform/waveform.h"
+
 #include <QtDebug>
 
-#include "waveform/waveform.h"
+#include "analyzer/constants.h"
+#include "engine/engine.h"
 #include "proto/waveform.pb.h"
 
 using namespace mixxx::track;
 
-const int kNumChannels = 2;
-
 // Return the smallest power of 2 which is greater than the desired size when
 // squared.
 int computeTextureStride(int size) {
@@ -17,9 +18,9 @@ int computeTextureStride(int size) {
     return stride;
 }
 
-Waveform::Waveform(const QByteArray data)
+Waveform::Waveform(const QByteArray& data)
         : m_id(-1),
-          m_bDirty(true),
+          m_saveState(SaveState::NotSaved),
           m_dataSize(0),
           m_visualSampleRate(0),
           m_audioVisualRatio(0),
@@ -28,10 +29,13 @@ Waveform::Waveform(const QByteArray data)
     readByteArray(data);
 }
 
-Waveform::Waveform(int audioSampleRate, int audioSamples,
-                   int desiredVisualSampleRate, int maxVisualSamples)
+Waveform::Waveform(
+        int audioSampleRate,
+        SINT frameLength,
+        int desiredVisualSampleRate,
+        int maxVisualSamples)
         : m_id(-1),
-          m_bDirty(true),
+          m_saveState(SaveState::NotSaved),
           m_dataSize(0),
           m_visualSampleRate(0),
           m_audioVisualRatio(0),
@@ -49,15 +53,18 @@ Waveform::Waveform(int audioSampleRate, int audioSamples,
             }
         } else {
             // Waveform Summary (Overview)
-            if (audioSamples > maxVisualSamples) {
-                m_visualSampleRate = (double)maxVisualSamples *
-                        (double)audioSampleRate / (double)audioSamples;
+            if (frameLength > maxVisualSamples / mixxx::kAnalysisChannels) {
+                m_visualSampleRate = static_cast<double>(audioSampleRate) *
+                        maxVisualSamples / mixxx::kAnalysisChannels / frameLength;
             } else {
                 m_visualSampleRate = audioSampleRate;
             }
         }
         m_audioVisualRatio = (double)audioSampleRate / (double)m_visualSampleRate;
-        numberOfVisualSamples = (audioSamples / m_audioVisualRatio) + 1;
+        numberOfVisualSamples =
+                static_cast<int>(frameLength / m_audioVisualRatio *
+                        mixxx::kAnalysisChannels) +
+                1;
         numberOfVisualSamples += numberOfVisualSamples%2;
     }
     assign(numberOfVisualSamples, 0);
@@ -74,7 +81,7 @@ QByteArray Waveform::toByteArray() const {
 
     io::Waveform::Signal* all = waveform.mutable_signal_all();
     io::Waveform::FilteredSignal* filtered = waveform.mutable_signal_filtered();
-    // TODO(rryan) get the actual cutoff values from analyserwaveform.cpp so
+    // TODO(rryan) get the actual cutoff values from analyzerwaveform.cpp so
     // that if they change we don't have to remember to update these.
 
     // Frequency cutoffs for butterworth filters:
@@ -96,15 +103,14 @@ QByteArray Waveform::toByteArray() const {
     io::Waveform::Signal* high = filtered->mutable_high();
 
     // TODO(vrince) set max/min for each signal
-    int numChannels = kNumChannels;
     all->set_units(io::Waveform::RMS);
-    all->set_channels(numChannels);
+    all->set_channels(mixxx::kEngineChannelCount);
     low->set_units(io::Waveform::RMS);
-    low->set_channels(numChannels);
+    low->set_channels(mixxx::kEngineChannelCount);
     mid->set_units(io::Waveform::RMS);
-    mid->set_channels(numChannels);
+    mid->set_channels(mixxx::kEngineChannelCount);
     high->set_units(io::Waveform::RMS);
-    high->set_channels(numChannels);
+    high->set_channels(mixxx::kEngineChannelCount);
 
     int dataSize = getDataSize();
     for (int i = 0; i < dataSize; ++i) {
@@ -123,7 +129,7 @@ QByteArray Waveform::toByteArray() const {
 
     std::string output;
     waveform.SerializeToString(&output);
-    return QByteArray(output.data(), output.length());
+    return QByteArray(output.data(), static_cast<int>(output.length()));
 }
 
 void Waveform::readByteArray(const QByteArray& data) {
@@ -167,6 +173,7 @@ void Waveform::readByteArray(const QByteArray& data) {
         qDebug() << "ERROR: Couldn't resize Waveform to" << all.value_size()
                  << "while reading.";
         resize(0);
+        m_saveState = SaveState::NotSaved;
         return;
     }
 
@@ -193,21 +200,20 @@ void Waveform::readByteArray(const QByteArray& data) {
         m_data[i].filtered.high = use_high ? static_cast<unsigned char>(high.value(i)) : 0;
     }
     m_completion = dataSize;
-    m_bDirty = false;
+    m_saveState = SaveState::Saved;
 }
 
 void Waveform::resize(int size) {
     m_dataSize = size;
     m_textureStride = computeTextureStride(size);
     m_data.resize(m_textureStride * m_textureStride);
-    m_bDirty = true;
 }
 
 void Waveform::assign(int size, int value) {
     m_dataSize = size;
     m_textureStride = computeTextureStride(size);
     m_data.assign(m_textureStride * m_textureStride, value);
-    m_bDirty = true;
+    m_saveState = SaveState::SavePending;
 }
 
 void Waveform::dump() const {
