@@ -1,16 +1,18 @@
-// wtracktableviewheader.cpp
-// Created 1/2/2010 by RJ Ryan (rryan@mit.edu)
-
-#include <QtDebug>
-
 #include "widget/wtracktableviewheader.h"
+
+#include <QCheckBox>
+#include <QContextMenuEvent>
+#include <QWidgetAction>
+
 #include "library/trackmodel.h"
+#include "moc_wtracktableviewheader.cpp"
 #include "util/math.h"
+#include "util/parented_ptr.h"
+#include "widget/wmenucheckbox.h"
 
 #define WTTVH_MINIMUM_SECTION_SIZE 20
 
-HeaderViewState::HeaderViewState(const QHeaderView& headers)
-{
+HeaderViewState::HeaderViewState(const QHeaderView& headers) {
     QAbstractItemModel* model = headers.model();
     for (int vi = 0; vi < headers.count(); ++vi) {
         int li = headers.logicalIndex(vi);
@@ -38,20 +40,22 @@ HeaderViewState::HeaderViewState(const QHeaderView& headers)
 }
 
 HeaderViewState::HeaderViewState(const QString& base64serialized) {
-    QByteArray array;
-    array.append(base64serialized);
     // First decode the array from Base64, then initialize the protobuf from it.
-    array = QByteArray::fromBase64(array);
+    QByteArray array = QByteArray::fromBase64(base64serialized.toLatin1());
     if (!m_view_state.ParseFromArray(array.constData(), array.size())) {
-        qDebug() << "ERROR: Could not parse m_view_state from QByteArray of size "
-                 << array.size();
+        qWarning() << "Could not parse m_view_state from QByteArray of size "
+                   << array.size();
         return;
     }
 }
 
 QString HeaderViewState::saveState() const {
     // Serialize the proto to a byte array, then encode the array as Base64.
+#if GOOGLE_PROTOBUF_VERSION >= 3001000
+    int size = static_cast<int>(m_view_state.ByteSizeLong());
+#else
     int size = m_view_state.ByteSize();
+#endif
     QByteArray array(size, '\0');
     m_view_state.SerializeToArray(array.data(), size);
     return QString(array.toBase64());
@@ -71,10 +75,9 @@ void HeaderViewState::restoreState(QHeaderView* headers) {
     // First set all sections to be hidden and update logical indexes.
     for (int li = 0; li < headers->count(); ++li) {
         headers->setSectionHidden(li, true);
-        // TODO(owilliams): replace with auto once we're building on c++11.
-        state_map::iterator it = map.find(
-                headers->model()->headerData(
-                        li, Qt::Horizontal, TrackModel::kHeaderNameRole).toString());
+        auto it = map.find(headers->model()->headerData(
+                                                   li, Qt::Horizontal, TrackModel::kHeaderNameRole)
+                                   .toString());
         if (it != map.end()) {
             it.value()->set_logical_index(li);
         }
@@ -99,16 +102,11 @@ void HeaderViewState::restoreState(QHeaderView* headers) {
 WTrackTableViewHeader::WTrackTableViewHeader(Qt::Orientation orientation,
                                              QWidget* parent)
         : QHeaderView(orientation, parent),
-          m_menu(tr("Show or hide columns."), this),
-          m_signalMapper(this) {
-    connect(&m_signalMapper, SIGNAL(mapped(int)),
-            this, SLOT(showOrHideColumn(int)));
-}
-
-WTrackTableViewHeader::~WTrackTableViewHeader() {
+          m_menu(tr("Show or hide columns."), this) {
 }
 
 void WTrackTableViewHeader::contextMenuEvent(QContextMenuEvent* event) {
+    event->accept();
     m_menu.popup(event->globalPos());
 }
 
@@ -144,13 +142,21 @@ void WTrackTableViewHeader::setModel(QAbstractItemModel* model) {
     restoreHeaderState();
 
     // Here we can override values to prevent restoring corrupt values from database
-    setMovable(true);
+    setSectionsMovable(true);
 
     // Setting true in the next line causes Bug #925619 at least with Qt 4.6.1
     setCascadingSectionResizes(false);
 
     setMinimumSectionSize(WTTVH_MINIMUM_SECTION_SIZE);
 
+    // Create a checkbox for each column.
+    // We want to keep the menu open after un/ticking a box because that allows
+    // to toggle multiple columns in one go, i.e. without having to open the
+    // menu again and again. This does not work with regular QActions so we
+    // create QCheckboxes inside QWidgetAction.
+    // * toggle a box with mouse click or Space on a selected box (via keyboard,
+    //   not just hovered by mouse pointer)
+    // * toggle and close by pressing Return on a selected box
     int columns = model->columnCount();
     for (int i = 0; i < columns; ++i) {
         if (trackModel->isColumnInternal(i)) {
@@ -158,28 +164,39 @@ void WTrackTableViewHeader::setModel(QAbstractItemModel* model) {
         }
 
         QString title = model->headerData(i, orientation()).toString();
-        QAction* action = new QAction(title, &m_menu);
-        action->setCheckable(true);
 
-        /* If Mixxx starts the first time or the header states have been cleared
-         * due to database schema evolution we gonna hide all columns that may
-         * contain a potential large number of NULL values.  Here we uncheck
-         * item in the context menu that are hidden by defualt (e.g., key
-         * column)
-         */
-        if (!hasPersistedHeaderState() &&
-            trackModel->isColumnHiddenByDefault(i)) {
-            action->setChecked(false);
+        // Custom QCheckBox with fixed hover behavior
+        auto pCheckBox = make_parented<WMenuCheckBox>(title, &m_menu);
+        // Keep a map of checkboxes and columns
+        m_columnCheckBoxes.insert(i, pCheckBox.get());
+        connect(pCheckBox.get(),
+                &QCheckBox::toggled,
+                this,
+                [this, i] {
+                    showOrHideColumn(i);
+                });
+        // If Mixxx starts the first time or the header states have been cleared
+        // due to database schema evolution we gonna hide all columns that may
+        // contain a potential large number of NULL values.  Here we uncheck
+        // the items that are hidden by default (e.g., key column).
+        if (!hasPersistedHeaderState() && trackModel->isColumnHiddenByDefault(i)) {
+            pCheckBox->setChecked(false);
         } else {
-            action->setChecked(!isSectionHidden(i));
+            pCheckBox->setChecked(!isSectionHidden(i));
         }
 
-        // Map this action's signals via our QSignalMapper
-        m_signalMapper.setMapping(action, i);
-        m_columnActions.insert(i, action);
-        connect(action, SIGNAL(triggered()),
-                &m_signalMapper, SLOT(map()));
-        m_menu.addAction(action);
+        auto pAction = make_parented<QWidgetAction>(this);
+        pAction->setDefaultWidget(pCheckBox.get());
+        // Pressing Return triggers the action but that would not toggle the
+        // checkbox, we need to do this ourselves while the menu is being closed.
+        connect(pAction,
+                &QAction::triggered,
+                this,
+                [this, pCheckBox{pCheckBox.get()}, i] {
+                    pCheckBox->toggle();
+                    showOrHideColumn(i);
+                });
+        m_menu.addAction(pAction);
 
         // force the section size to be a least WTTVH_MINIMUM_SECTION_SIZE
         if (sectionSize(i) <  WTTVH_MINIMUM_SECTION_SIZE) {
@@ -250,50 +267,46 @@ bool WTrackTableViewHeader::hasPersistedHeaderState() {
         return false;
     }
     QString headerStateString = track_model->getModelSetting("header_state_pb");
-    if (!headerStateString.isNull()) {
-        return true;
-    }
-    return false;
+    return !headerStateString.isNull();
 }
 
 void WTrackTableViewHeader::clearActions() {
     // The QActions are parented to the menu, so clearing deletes them. Since
     // they are deleted we don't have to disconnect their signals from the
     // mapper.
-    m_columnActions.clear();
+    m_columnCheckBoxes.clear();
     m_menu.clear();
 }
 
 void WTrackTableViewHeader::showOrHideColumn(int column) {
-    if (!m_columnActions.contains(column)) {
-        qDebug() << "WTrackTableViewHeader got invalid column" << column;
+    auto it = m_columnCheckBoxes.constFind(column);
+    if (it == m_columnCheckBoxes.constEnd()) {
+        qWarning() << "WTrackTableViewHeader got invalid column" << column;
         return;
     }
-
-    QAction* action = m_columnActions[column];
-    if (action->isChecked()) {
+    QCheckBox* pCheckBox = it.value();
+    if (pCheckBox->isChecked()) {
         showSection(column);
     } else {
         // If the user hides every column then the table will disappear. This
         // guards against that. NB: hiddenCount reflects checked QAction's so
         // size-hiddenCount will be zero the moment they uncheck the last
         // section.
-        if (m_columnActions.size() - hiddenCount() > 0) {
+        if (m_columnCheckBoxes.size() - hiddenCount() > 0) {
             hideSection(column);
         } else {
             // Otherwise, ignore the request and re-check this QAction.
-            action->setChecked(true);
+            pCheckBox->setChecked(true);
         }
     }
 }
 
 int WTrackTableViewHeader::hiddenCount() {
     int count = 0;
-    for (QMap<int, QAction*>::iterator it = m_columnActions.begin();
-         it != m_columnActions.end(); ++it) {
-        QAction* pAction = *it;
-        if (!pAction->isChecked())
+    for (const auto& pCheckBox : std::as_const(m_columnCheckBoxes)) {
+        if (!pCheckBox->isChecked()) {
             count += 1;
+        }
     }
     return count;
 }
